#!/usr/bin/perl -w

use strict;
use warnings;
use Inca::DummyServer;
use SOAP::Lite;# +'trace';
use Test::More 'no_plan';

my $AGENT_PORT = 5678;
my $DEPOT_PORT = 6789;
my $WEB_SERVICE_PORT = 8001;
my $WEB_SERVICE_PATH = "../bin/incaws";
chomp(my $WSDL_URL = `pwd`);
$WSDL_URL =~ s#[^/]+$#etc/IncaWS.wsdl#;
$WSDL_URL = "file://$WSDL_URL";
my $REPOSITORY_URL = 'http://inca.sdsc.edu/2.0/repository/';
my $GET_CONFIG_REPLY =
"<inca:inca xmlns:inca='http://inca.sdsc.edu/dataModel/inca_2.0'>
  <repositories>
    <repository>$REPOSITORY_URL</repository>
  </repositories>
  <resourceConfig>
    <resources>
      <resource>
        <name>blue.ufo.edu</name>
      </resource>
      <resource>
        <name>green.ufo.edu</name>
      </resource>
      <resource>
        <name>red.ufo.edu</name>
      </resource>
    </resources>
  </resourceConfig>
</inca:inca>";
my $QUERY_GUIDS_REPLY = "guid1 guid2 guid3";
my @QUERY_HQL_REPLY = (
'<instance>
  <collected>2006-10-09 09:10:03.0</collected>
  <committed>2006-10-09 09:10:03.927</committed>
  <cpu>0.16</cpu>
  <memory>0.0</memory>
  <wall>0.365195</wall>
</instance>
',
'<instance>
  <collected>2006-10-09 09:10:04.0</collected>
  <committed>2006-10-09 09:10:05.007</committed>
  <cpu>0.69</cpu>
  <memory>9.1171875</memory>
  <wall>1.752196</wall>
</instance>
',
'<instance>
  <collected>2006-10-09 09:15:02.0</collected>
  <committed>2006-10-09 09:15:04.506</committed>
  <cpu>0.15</cpu>
  <memory>0.0</memory>
  <wall>0.416581</wall>
</instance>
');
my $QUERY_INSTANCE_REPLY =
'<reportDetails xmlns="http://inca.sdsc.edu/queryResult/reportDetails_2.0">
  <suiteId xmlns="">1</suiteId>
  <seriesConfigId xmlns="">1</seriesConfigId>
  <seriesId xmlns="">1</seriesId>
  <reportId xmlns="">1</reportId>
  <instanceId xmlns="">39</instanceId>
  <seriesConfig xmlns="">
    <series>
      <name>cluster.compiler.gcc.version</name>
      <version>1.5</version>
      <uri>file:///home/inca/reporters/cluster.compiler.gcc.version</uri>
      <args>
        <arg>
          <name>version</name>
          <value>no</value>
        </arg>
        <arg>
          <name>help</name>
          <value>no</value>
        </arg>
        <arg>
          <name>log</name>
          <value>0</value>
        </arg>
        <arg>
          <name>verbose</name>
          <value>1</value>
        </arg>
      </args>
      <context>cluster.compiler.gcc.version -help="no" -log="0" -verbose="1" -version="no"</context>
      <nice>false</nice>
    </series>
    <nickname>cluster.compiler.gcc.version</nickname>
    <resourceHostname>localhost</resourceHostname>
    <schedule>
      <cron>
        <min>*/5</min>
        <hour>*</hour>
        <mday>*</mday>
        <wday>*</wday>
        <month>*</month>
        <numOccurs>-1</numOccurs>
        <suspended>false</suspended>
      </cron>
    </schedule>
    <action>add</action>
  </seriesConfig>
  <report xmlns="">
    <gmt>2006-10-09T10:45:03.000-07:00</gmt>
    <hostname>client64-120.sdsc.edu</hostname>
    <name>cluster.compiler.gcc.version</name>
    <version>1.5</version>
    <workingDir>/private/tmp/jjhinstall</workingDir>
    <reporterPath>/private/tmp/jjhinstall/var/reporter-packages/bin/cluster.compiler.gcc.version-1.5</reporterPath>
    <args>
      <arg>
        <name>version</name>
        <value>no</value>
      </arg>
      <arg>
        <name>help</name>
        <value>no</value>
      </arg>
      <arg>
        <name>log</name>
        <value>0</value>
      </arg>
      <arg>
        <name>verbose</name>
        <value>1</value>
      </arg>
    </args>
    <body>
      <package xmlns:rep="http://inca.sdsc.edu/dataModel/report_2.1">
        <ID>gcc</ID>
        <version>3.3</version>
      </package>
    </body>
    <exitStatus>
      <completed>true</completed>
      <errorMessage/>
    </exitStatus>
  </report>
  <sysusage xmlns="">
    <wallClockTime>0.321577</wallClockTime>
    <memory>0.0</memory>
    <cpuTime>0.11</cpuTime>
  </sysusage>
  <stderr xmlns=""/>
</reportDetails>';
my @QUERY_SERIES_REPLY = ($QUERY_INSTANCE_REPLY);
my @QUERY_SUITE_REPLY = (
'<reportSummary xmlns="http://inca.sdsc.edu/queryResult/reportSummary_2.0">
  <hostname xmlns="">localhost</hostname>
  <uri xmlns="">file:///home/inca/reporters/bin/cluster.compiler.gcc.version</uri>
  <nickname xmlns="">cluster.compiler.gcc.version</nickname>
  <seriesConfigId xmlns="">1</seriesConfigId>
  <instanceId xmlns="">5</instanceId>  <body xmlns="">
    <package xmlns:rep="http://inca.sdsc.edu/dataModel/report_2.1">
      <ID>gcc</ID>
      <version>3.3</version>
    </package>
  </body>
</reportSummary>',
'<reportSummary xmlns="http://inca.sdsc.edu/queryResult/reportSummary_2.0">
  <hostname xmlns="">localhost</hostname>
  <uri xmlns="">file:///home/inca/reporters/bin/cluster.admin.ant.version</uri>
  <nickname xmlns="">cluster.admin.ant.version</nickname>
  <seriesConfigId xmlns="">2</seriesConfigId>
  <instanceId xmlns="">6</instanceId>
  <body xmlns="">
    <package xmlns:rep="http://inca.sdsc.edu/dataModel/report_2.1">
      <ID>ant</ID>
      <version>1.6.5</version>
    </package>
  </body>
</reportSummary>'
);
my $aCat = 'name:r1\nversion:1.0\n\nname:r2\nversion:3.7\n';
my $bCat = 'name:r3\nversion:9.7\n\nname:r4\nversion:3beta\n';

my $dummyAgentPid = fork();
if($dummyAgentPid == 0) {
  my $agent = new Inca::DummyServer($AGENT_PORT,
    'GETCATALOG (.+)' => "q#OK $aCat#",
    'GETCATALOG'      => "q#OK $aCat\n$bCat#",
    'GETCONFIG'       => "q#OK $GET_CONFIG_REPLY#",
    'PING (.*)'       => 'qq#OK $1#',
    'START (.+)'      => 'qq#OK $1#'
  );
  $agent->run();
  exit(1);
}
my $dummyDepotPid = fork();
if($dummyDepotPid == 0) {
  my $depot = new Inca::DummyServer($DEPOT_PORT,
    'QUERYGUIDS'         => "q#OK $QUERY_GUIDS_REPLY#",
    'QUERYHQL (.+)'      =>
      "q#QUERYRESULT " .
      join("\r\nQUERYRESULT ", @QUERY_HQL_REPLY) .
      "\r\nQUERYEND#",
    'QUERYINSTANCE (\d+) (\d+)' =>
      "q#QUERYRESULT $QUERY_INSTANCE_REPLY\r\nQUERYEND#",
    'QUERYSERIES (\d+)'   =>
      "q#QUERYRESULT " .
      join("\r\nQUERYRESULT ", @QUERY_SERIES_REPLY) .
      "\r\nQUERYEND#",
    'QUERYSQL (.+)'      => "q#ERROR TODO'",
    'QUERYSUITE (.+)'    =>
      "q#QUERYRESULT " .
      join("\r\nQUERYRESULT ", @QUERY_SUITE_REPLY) .
      "\r\nQUERYEND#",
    'PING (.+)'          => 'qq#OK $1#',
    'START (.+)'         => 'qq#OK $1#'
  );
  $depot->run();
  exit(1);
}

my $PING1 = "Hi Agent";
my $PING2 = "Hi Depot";
my @CALLS = (
  'getCatalog("")',
  'getCatalog("file:///home/inca/reporters/bin/")',
  'getConfig()',
  "pingAgent('$PING1')",
  "pingDepot('$PING2')",
  'queryGuids()',
  'queryHql("select s from Suite as s")',
  "queryInstance(1, 2)",
  "querySeries(2)",
  "querySuite('suiteguid')",
);
my @EXPECTED_REPLIES = (
  "$aCat\n$bCat",
  $aCat,
  $GET_CONFIG_REPLY,
  'Hi Agent',
  'Hi Depot',
  $QUERY_GUIDS_REPLY,
  '["' . join('", "', @QUERY_HQL_REPLY) . '"]',
  '["' . $QUERY_INSTANCE_REPLY . '"]',
  '["' . join('", "', @QUERY_SERIES_REPLY) . '"]',
  '["' . join('", "', @QUERY_SUITE_REPLY) . '"]'
);

# Start the web service and open two kinds of connections
# Establish connections to the agent, depot, and web service
my $webServicePid = fork();
if($webServicePid == 0) {
  exec("perl -I ../lib $WEB_SERVICE_PATH --auth=no --password pass:mypass --port $WEB_SERVICE_PORT localhost:$DEPOT_PORT localhost:$AGENT_PORT");
  exit(1);
}
my $wsClient =
  SOAP::Lite->uri('urn:IncaWebService')->proxy("http://localhost:$WEB_SERVICE_PORT");
my $wsStubs = SOAP::Lite->service($WSDL_URL);

# Finally, try some tests
for(my $i = 0; $i <= $#CALLS; $i++) {
  my $call = $CALLS[$i];
  my $invocation = '$wsClient->' . $call;
  my $expected = $EXPECTED_REPLIES[$i];
  my $response = eval $invocation;
  my $result;
  if(!defined($response) || $response->fault) {
    &fail($invocation);
    print join(', ', $response->faultcode(), $response->faultstring())
      if defined($response);
  } else {
    $result = $response->result();
    $result = '' if !defined $result;
    if(ref($result) && ref($result) eq 'ARRAY') {
      $result =
        scalar(@$result) > 0 ? '["' . join('", "', @$result) . '"]' : '[]';
    }
    my @params = $response->paramsout();
    $result = "$result @params" if $#params >= 0;
    if($result ne $expected) {
      print STDERR "Expected '$expected', got '$result'\n";
    }
    &ok($result eq $expected, $call);
  }
  $invocation = '$wsStubs->' . $call;
  $result = eval $invocation;
  $result = '' if !defined $result;
  if(ref($result) && ref($result) eq 'ARRAY') {
    $result =
      scalar(@$result) > 0 ? '["' . join('", "', @$result) . '"]' : '[]';
  }
  if($result ne $expected) {
    print STDERR "Expected '$expected', got '$result'\n";
  }
  &ok($result eq $expected, $call);
}

kill 9, $dummyAgentPid;
kill 9, $dummyDepotPid;
kill 9, $webServicePid;
